# Copyright (C) 2016  Pachol, Vojtěch <pacholick@gmail.com>
#
# This program is free software: you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

import re
import math

from commacalc.extra import sign


_SYMBOLS = {
    'k': "1000",
    'M': "1000000",
    'G': "1000000000",
    'T': "1000000000000",
    'P': "1000000000000000",

    'm': "0.001",
    'μ': "0.000001",
    'n': "0.000000001",
    'p': "0.000000000001",
    'f': "0.000000000000001",

    '°': "(π/180)",
}


def replace_symbols(input_):
    """replace 123k with 1000*123"""
    for symbol, value in _SYMBOLS.items():
        input_ = re.sub(
            r'(\d+){}'.format(symbol),
            r'\1*{}'.format(value),
            input_
        )
    return input_


def from_fahrenheit(fahrenheit):
    """Convert Fahrenheits to Centigrade."""
    return (fahrenheit - 32)*5/9


C = from_fahrenheit


def to_fahrenheit(c):
    """Convert Centigrade to Fahrenheits."""
    return c*9/5 + 32


def prefix(x, dimension=1):
    """Give the number an appropriate SI prefix.

    :param x:   Too big or too small number.
    :returns:   String containing a number between 1 and 1000 and SI prefix.
    """
    if x == 0:
        return "0  "

    l = math.floor(math.log10(abs(x)))
    if abs(l) > 24:
        l = sign(l, value=24)

    div, mod = divmod(l, 3*dimension)
    # return "%.3g %s" % (x * 10**(-l + l % 3), " kMGTPEZYyzafpnµm"[l//3])
    return "{:.3g} {}".format(x*10**(-l + mod), " kMGTPEZYyzafpnµm"[div])


# maths
π = math.pi
φ = (1 + math.sqrt(5))/2

# physics
c = 299792458
"""Speed of light in vacuum (exact) [ m s⁻¹ ]"""
G = 6.6738480e-11
"""Gravitational constant [ m³ kg⁻¹ s⁻² ]"""
h = 6.6260695729e-34
"""Planck constant [ J s ]"""
ħ = h/(2*math.pi)
"""Reduced Planck constant [ J s ]"""
k_e = 8.9875517873681764e9
"""Coulomb constant (exact) [ kg m³ s⁻² C⁻² ]"""
k_B = 1.380648813e-23
"""Boltzmann constant [ J K⁻¹ ]"""

eV = 1.60217656535e-19
"""Electronvolt"""


# imperial units to metres (exact)
inch = 0.0254
foot = ft = 0.3048
yard = yd = 0.9144
mile = mi = 1609.344
acre = ac = 4046.8564224
pound = lb = 0.45359237
barrel = 0.15897    # (not exact)


# time
days_in_year = 365.242189
seconds_in_day = 86400
seconds_in_year = days_in_year*seconds_in_day


if __name__ == "__main__":
    print(prefix(0.9, 2))



# Copyright (C) 2016  Pachol, Vojtěch <pacholick@gmail.com>
#
# This program is free software: you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

import math

import commacalc.units


def cbrt(x):
    """Return the cube root of x."""
    return x**(1/3)


def dsp_wh(ratio, diag):
    """Find width and height of a screen, given its ratio and diagonal.

    :param ratio:   Ratio of a screen (e.g. 16/9)
    :param diag:    Diagonal of a screen in inches

    :returns:       Width and Height of a screen in metres
    """
    cosinus = math.cos(math.atan(ratio))
    h = diag * cosinus * commacalc.units.inch * 100
    w = h * ratio
    return w, h


def sign(x, value=1):
    """Mathematical signum function.

    :param x:       Object of investigation
    :param value:   The size of the signum (defaults to 1)

    :returns:       Plus or minus value
    """
    return -value if x < 0 else value


def isqrt(n):
    """isqrt(n)

    :returns:   floor(sqrt(n))
    """
    if not isinstance(n, int):
        raise TypeError('an int is required')
    if n < 0:
        raise ValueError('math domain error')

    guess = (n >> n.bit_length() // 2) + 1
    result = (guess + n // guess) // 2
    while abs(result - guess) > 1:
        guess = result
        result = (guess + n // guess) // 2
    while result * result > n:
        result -= 1
    return result


def nCk(n, k):
    f = math.factorial
    return f(n) // (f(k) * f(n-k))



# Copyright (C) 2016  Pachol, Vojtěch <pacholick@gmail.com>
#
# This program is free software: you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

import sys
import re
from math import *                      # noqa: F401, F403
import numbers

import commacalc.currencies
import commacalc.units
import commacalc.pretty_number

from commacalc.currencies import *      # noqa: F401, F403
from commacalc.units import *           # noqa: F401, F403
from commacalc.extra import *           # noqa: F401, F403
# from commacalc.primes import *


_FUNCTIONS = {
    '√': "sqrt",
    '∛': "cbrt",
}


def replace_functions(input_):
    """replace √123 or √(123) with sqrt(123)"""
    for symbol, value in _FUNCTIONS.items():
        input_ = re.sub(
            r'{}([\d.]+)'.format(symbol),
            r'{}(\1)'.format(value),
            input_
        )
    for symbol, value in _FUNCTIONS.items():
        input_ = re.sub(
            r'{}\('.format(symbol),
            r'{}('.format(value),
            input_
        )
    return input_


def main():
    input_ = ''.join(sys.argv[1:])

    input_ = replace_functions(input_)
    input_ = commacalc.currencies.replace_symbols(input_)
    input_ = commacalc.units.replace_symbols(input_)

    result = eval(input_)
    if isinstance(result, numbers.Real):
        result = commacalc.pretty_number.PrettyNumber(result)
    print(result)


if __name__ == '__main__':
    sys.argv = [',', '4*7']
    main()



# Copyright (C) 2016  Pachol, Vojtěch <pacholick@gmail.com>
#
# This program is free software: you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

import os
import pickle
from stat import ST_MTIME
from time import time
import threading


_SYMBOLS = {
    '$': "USD",
    '£': "GBP",
    '€': "EUR",
    '¥': "JPY",
}


def replace_symbols(input_):
    """replace €123 with EUR*123"""
    for symbol, code in _SYMBOLS.items():
        input_ = input_.replace(symbol, code+'*')
    return input_


RATES_URLS = [
    ("https://www.cnb.cz/cs/financni_trhy/devizovy_trh/"
     "kurzy_devizoveho_trhu/denni_kurz.txt"),
    ("https://www.cnb.cz/cs/financni_trhy/devizovy_trh/"
     "kurzy_ostatnich_men/kurzy.txt")
]
RATES_PICKLE = os.path.expanduser("~/.rates.pickle")

rates = {}


def _download():
    import httplib2
    import csv
    import io

    h = httplib2.Http()

    for url in RATES_URLS:
        response, content = h.request(url)

        reader = csv.reader(io.StringIO(content.decode()), delimiter='|')
        next(reader)    # header
        next(reader)    # column names
        rates.update({
            currcode: float(rate.replace(',', '.'))/int(amount) for
            country, currname, amount, currcode, rate in reader
        })


def _dump():
    with open(RATES_PICKLE, mode="wb") as f:
        pickle.dump(rates, f)


def _dl_n_dump():
    _download()
    _dump()


def rate(currency):
    """Get exchange rate (ČNB)

    :param currency:    Code of currency (e.g. "USD" or "EUR")
    :returns:           Exchange rate in CZK
    """
    global rates

    if not rates:
        if not os.path.exists(RATES_PICKLE):
            _download()
            threading.Thread(target=_dump).start()
        elif time() - os.stat(RATES_PICKLE)[ST_MTIME] > 86400:
            with open(RATES_PICKLE, mode="rb") as f:
                rates = pickle.load(f)
            threading.Thread(target=_dl_n_dump).start()
        else:
            with open(RATES_PICKLE, mode="rb") as f:
                rates = pickle.load(f)

    return rates[currency]


# curencies
USD = dolar = rate("USD")
EUR = euro = rate("EUR")
GBP = libra = rate("GBP")
RUB = rubl = rate("RUB")
JPY = jen = rate("JPY")


if __name__ == "__main__":
    for c in "USD", "EUR", "VND":
        print("{}: {}".format(c, rate(c)))
